@@ -246,6 +246,29 @@ _as_app_quirk_flag_to_string (AsAppQuirk quirk)
 	return NULL;
 }
 
+/* mutex must be held */
+static const gchar *
+gs_app_get_unique_id_unlocked (GsApp *app)
+{
+	/* invalid */
+	if (app->id == NULL)
+		return NULL;
+
+	/* hmm, do what we can */
+	if (app->unique_id == NULL || !app->unique_id_valid) {
+		g_debug ("autogenerating unique-id for %s", app->id);
+		g_free (app->unique_id);
+		app->unique_id = as_utils_unique_id_build (app->scope,
+							   app->bundle_kind,
+							   app->origin,
+							   app->kind,
+							   app->id,
+							   app->branch);
+		app->unique_id_valid = TRUE;
+	}
+	return app->unique_id;
+}
+
 /**
  * _as_app_quirk_to_string:
  * @quirk: a #AsAppQuirk
@@ -751,10 +774,15 @@ gs_app_set_state_recover (GsApp *app)
 		 as_app_state_to_string (app->state),
 		 as_app_state_to_string (app->state_recover));
 
+	/* make sure progress gets reset when recovering state, to prevent
+	 * confusing initial states when going through more than one attempt */
+	gs_app_set_progress (app, 0);
+
 	app->state = app->state_recover;
 	gs_app_queue_notify (app, "state");
 }
 
+/* mutex must be held */
 static gboolean
 gs_app_set_state_internal (GsApp *app, AsAppState state)
 {
@@ -824,6 +852,7 @@ gs_app_set_state_internal (GsApp *app, AsAppState state)
 	case AS_APP_STATE_UPDATABLE_LIVE:
 		/* updatable-live has to go into an action state */
 		if (state == AS_APP_STATE_UNKNOWN ||
+		    state == AS_APP_STATE_REMOVING ||
 		    state == AS_APP_STATE_INSTALLING)
 			state_change_ok = TRUE;
 		break;
@@ -848,7 +877,7 @@ gs_app_set_state_internal (GsApp *app, AsAppState state)
 	/* this state change was unexpected */
 	if (!state_change_ok) {
 		g_warning ("State change on %s from %s to %s is not OK",
-			   gs_app_get_unique_id (app),
+			   gs_app_get_unique_id_unlocked (app),
 			   as_app_state_to_string (app->state),
 			   as_app_state_to_string (state));
 		return FALSE;
@@ -870,7 +899,7 @@ gs_app_set_state_internal (GsApp *app, AsAppState state)
 	default:
 		if (app->state_recover != state) {
 			g_debug ("%s non-transient state now %s",
-				 gs_app_get_unique_id (app),
+				 gs_app_get_unique_id_unlocked (app),
 				 as_app_state_to_string (state));
 			app->state_recover = state;
 		}
@@ -894,13 +923,14 @@ gs_app_set_state_internal (GsApp *app, AsAppState state)
 void
 gs_app_set_progress (GsApp *app, guint percentage)
 {
+	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&app->mutex);
 	g_return_if_fail (GS_IS_APP (app));
 	if (app->progress == percentage)
 		return;
 	if (percentage > 100) {
 		g_warning ("Cannot set '%u' as the progress for app '%s'. "
 			   "Setting the maximum allowed value instead: 100.",
-			   percentage, gs_app_get_unique_id (app));
+			   percentage, gs_app_get_unique_id_unlocked (app));
 		percentage = 100;
 	}
 	app->progress = percentage;
@@ -934,6 +964,7 @@ gs_app_set_progress (GsApp *app, guint percentage)
 void
 gs_app_set_state (GsApp *app, AsAppState state)
 {
+	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&app->mutex);
 	g_return_if_fail (GS_IS_APP (app));
 
 	if (gs_app_set_state_internal (app, state))
@@ -978,6 +1009,7 @@ void
 gs_app_set_kind (GsApp *app, AsAppKind kind)
 {
 	gboolean state_change_ok = FALSE;
+	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&app->mutex);
 
 	g_return_if_fail (GS_IS_APP (app));
 
@@ -990,7 +1022,7 @@ gs_app_set_kind (GsApp *app, AsAppKind kind)
 	    kind == AS_APP_KIND_UNKNOWN) {
 		g_warning ("automatically prevented from changing "
 			   "kind on %s from %s to %s!",
-			   gs_app_get_unique_id (app),
+			   gs_app_get_unique_id_unlocked (app),
 			   as_app_kind_to_string (app->kind),
 			   as_app_kind_to_string (kind));
 		return;
@@ -1045,24 +1077,7 @@ gs_app_get_unique_id (GsApp *app)
 {
 	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&app->mutex);
 	g_return_val_if_fail (GS_IS_APP (app), NULL);
-
-	/* invalid */
-	if (app->id == NULL)
-		return NULL;
-
-	/* hmm, do what we can */
-	if (app->unique_id == NULL || !app->unique_id_valid) {
-		g_debug ("autogenerating unique-id for %s", app->id);
-		g_free (app->unique_id);
-		app->unique_id = as_utils_unique_id_build (app->scope,
-							   app->bundle_kind,
-							   app->origin,
-							   app->kind,
-							   app->id,
-							   app->branch);
-		app->unique_id_valid = TRUE;
-	}
-	return app->unique_id;
+	return gs_app_get_unique_id_unlocked (app);
 }
 
 /**
@@ -1197,6 +1212,7 @@ gs_app_add_source (GsApp *app, const gchar *source)
 {
 	const gchar *tmp;
 	guint i;
+	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&app->mutex);
 
 	g_return_if_fail (GS_IS_APP (app));
 	g_return_if_fail (source != NULL);
@@ -1538,6 +1554,8 @@ gs_app_set_runtime (GsApp *app, GsApp *runtime)
 {
 	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&app->mutex);
 	g_return_if_fail (GS_IS_APP (app));
+	g_return_if_fail (app != runtime);
+	g_return_if_fail (runtime->kind == AS_APP_KIND_RUNTIME);
 	g_set_object (&app->runtime, runtime);
 }
 
@@ -1572,6 +1590,8 @@ gs_app_set_update_runtime (GsApp *app, GsApp *runtime)
 {
 	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&app->mutex);
 	g_return_if_fail (GS_IS_APP (app));
+	g_return_if_fail (app != runtime);
+	g_return_if_fail (runtime->kind == AS_APP_KIND_RUNTIME);
 	g_set_object (&app->update_runtime, runtime);
 }
 
@@ -2068,7 +2088,7 @@ gs_app_get_origin (GsApp *app)
 void
 gs_app_set_origin (GsApp *app, const gchar *origin)
 {
-	g_autoptr(GMutexLocker) locker = NULL;
+	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&app->mutex);
 	g_return_if_fail (GS_IS_APP (app));
 
 	/* same */
@@ -2079,12 +2099,11 @@ gs_app_set_origin (GsApp *app, const gchar *origin)
 	if (app->origin != NULL && origin != NULL) {
 		g_warning ("automatically prevented from changing "
 			   "origin on %s from %s to %s!",
-			   gs_app_get_unique_id (app),
+			   gs_app_get_unique_id_unlocked (app),
 			   app->origin, origin);
 		return;
 	}
 
-	locker = g_mutex_locker_new (&app->mutex);
 	g_free (app->origin);
 	app->origin = g_strdup (origin);
 
@@ -2185,6 +2204,10 @@ gs_app_set_origin_hostname (GsApp *app, const gchar *origin_hostname)
 			origin_hostname += strlen (prefixes[i]);
 	}
 
+	/* fallback for localhost */
+	if (g_strcmp0 (origin_hostname, "") == 0)
+		origin_hostname = "localhost";
+
 	/* success */
 	app->origin_hostname = g_strdup (origin_hostname);
 }
@@ -2404,7 +2427,7 @@ gs_app_set_management_plugin (GsApp *app, const gchar *management_plugin)
 	if (gs_app_has_quirk (app, AS_APP_QUIRK_MATCH_ANY_PREFIX)) {
 		g_warning ("plugins should not set the management plugin on "
 			   "%s to %s -- create a new GsApp in refine()!",
-			   gs_app_get_unique_id (app),
+			   gs_app_get_unique_id_unlocked (app),
 			   management_plugin);
 		return;
 	}
@@ -2417,7 +2440,7 @@ gs_app_set_management_plugin (GsApp *app, const gchar *management_plugin)
 	if (app->management_plugin != NULL && management_plugin != NULL) {
 		g_warning ("automatically prevented from changing "
 			   "management plugin on %s from %s to %s!",
-			   gs_app_get_unique_id (app),
+			   gs_app_get_unique_id_unlocked (app),
 			   app->management_plugin,
 			   management_plugin);
 		return;
@@ -2456,6 +2479,7 @@ gs_app_get_rating (GsApp *app)
 void
 gs_app_set_rating (GsApp *app, gint rating)
 {
+	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&app->mutex);
 	g_return_if_fail (GS_IS_APP (app));
 	if (rating == app->rating)
 		return;
@@ -3367,6 +3391,7 @@ gs_app_has_quirk (GsApp *app, AsAppQuirk quirk)
 void
 gs_app_add_quirk (GsApp *app, AsAppQuirk quirk)
 {
+	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&app->mutex);
 	g_return_if_fail (GS_IS_APP (app));
 
 	app->quirk |= quirk;
@@ -3385,6 +3410,7 @@ gs_app_add_quirk (GsApp *app, AsAppQuirk quirk)
 void
 gs_app_remove_quirk (GsApp *app, AsAppQuirk quirk)
 {
+	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&app->mutex);
 	g_return_if_fail (GS_IS_APP (app));
 
 	app->quirk &= ~quirk;
