@@ -130,18 +130,26 @@ void
 gs_app_notify_installed (GsApp *app)
 {
 	g_autofree gchar *summary = NULL;
+	const gchar *body = NULL;
 	g_autoptr(GNotification) n = NULL;
 
 	if (gs_app_get_kind (app) == AS_APP_KIND_OS_UPDATE) {
 		/* TRANSLATORS: this is the summary of a notification that OS updates
 		 * have been successfully installed */
 		summary = g_strdup (_("OS updates are now installed"));
+		/* TRANSLATORS: this is the body of a notification that OS updates
+		 * have been successfully installed */
+		body = _("Recently installed updates are available to review");
 	} else {
 		/* TRANSLATORS: this is the summary of a notification that an application
 		 * has been successfully installed */
 		summary = g_strdup_printf (_("%s is now installed"), gs_app_get_name (app));
+		/* TRANSLATORS: this is the body of a notification that an application
+		 * has been successfully installed */
+		body = _("Application is ready to be used.");
 	}
 	n = g_notification_new (summary);
+	g_notification_set_body (n, body);
 	if (gs_app_get_kind (app) == AS_APP_KIND_DESKTOP) {
 		/* TRANSLATORS: this is button that opens the newly installed application */
 		g_notification_add_button_with_target (n, _("Launch"),
@@ -305,6 +313,8 @@ gs_image_set_from_pixbuf_with_scale (GtkImage *image, const GdkPixbuf *pixbuf, g
 {
 	cairo_surface_t *surface;
 	surface = gdk_cairo_surface_create_from_pixbuf (pixbuf, scale, NULL);
+	if (surface == NULL)
+		return;
 	gtk_image_set_from_surface (image, surface);
 	cairo_surface_destroy (surface);
 }
@@ -564,53 +574,6 @@ gs_utils_show_error_dialog (GtkWindow *parent,
 	gtk_widget_show (dialog);
 }
 
-static void
-gs_search_button_cb (GtkToggleButton *toggle_button, GtkSearchBar *search_bar)
-{
-	gtk_search_bar_set_search_mode (search_bar,
-					gtk_toggle_button_get_active (toggle_button));
-}
-
-static void
-gs_search_mode_enabled_cb (GtkSearchBar *search_bar,
-			   GParamSpec *pspec,
-			   GtkToggleButton *toggle_button)
-{
-	gtk_toggle_button_set_active (toggle_button,
-				      gtk_search_bar_get_search_mode (search_bar));
-}
-
-/**
- * gs_search_button_new:
- *
- * Creates a new search toggle button.
- */
-GtkWidget *
-gs_search_button_new (GtkSearchBar *search_bar)
-{
-	GtkWidget *widget;
-	GtkWidget *im;
-
-	widget = gtk_toggle_button_new ();
-	im = gtk_image_new_from_icon_name ("edit-find-symbolic", GTK_ICON_SIZE_BUTTON);
-	gtk_widget_set_visible (im, TRUE);
-	gtk_container_add (GTK_CONTAINER (widget), im);
-	gtk_widget_set_visible (widget, TRUE);
-
-	if (search_bar != NULL) {
-		/* show the search bar when clicked */
-		g_signal_connect (widget, "clicked",
-				  G_CALLBACK (gs_search_button_cb),
-				  search_bar);
-		/* set the button enabled when search appears */
-		g_signal_connect (search_bar, "notify::search-mode-enabled",
-				  G_CALLBACK (gs_search_mode_enabled_cb),
-				  widget);
-	}
-
-	return widget;
-}
-
 /**
  * gs_utils_get_error_value:
  * @error: A GError
