@@ -671,7 +671,7 @@ bus_gotten_cb (GObject      *source_object,
 
 	connection = g_bus_get_finish (res, &error);
 	if (connection == NULL) {
-		if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+		if (!g_error_matches (error, GS_PLUGIN_ERROR, GS_PLUGIN_ERROR_CANCELLED))
 			g_warning ("Could not get session bus: %s", error->message);
 		return;
 	}
