@@ -54,9 +54,10 @@ void		 gs_page_set_header_start_widget	(GsPage		*page,
 GtkWidget	*gs_page_get_header_end_widget		(GsPage		*page);
 void		 gs_page_set_header_end_widget		(GsPage		*page,
 							 GtkWidget	*widget);
-void		 gs_page_install_app			(GsPage		*page,
-							 GsApp		*app,
-							 GCancellable	*cancellable);
+void		 gs_page_install_app			(GsPage			*page,
+							 GsApp			*app,
+							 GsShellInteraction	interaction,
+							 GCancellable		*cancellable);
 void		 gs_page_remove_app			(GsPage		*page,
 							 GsApp		*app,
 							 GCancellable	*cancellable);
@@ -79,7 +80,6 @@ void		 gs_page_setup				(GsPage		*page,
 							 GsShell	*shell,
 							 GsPluginLoader	*plugin_loader,
 							 GCancellable	*cancellable);
-gboolean	 gs_page_is_active			(GsPage		*page);
 
 G_END_DECLS
 
