@@ -47,6 +47,8 @@
 #include <gio/gdesktopappinfo.h>
 #include <gdk/gdk.h>
 
+#include <valgrind.h>
+
 #include "gs-app-list-private.h"
 #include "gs-os-release.h"
 #include "gs-plugin-private.h"
@@ -208,8 +210,10 @@ gs_plugin_finalize (GObject *object)
 	g_mutex_clear (&priv->cache_mutex);
 	g_mutex_clear (&priv->timer_mutex);
 	g_mutex_clear (&priv->vfuncs_mutex);
+#ifndef RUNNING_ON_VALGRIND
 	if (priv->module != NULL)
 		g_module_close (priv->module);
+#endif
 }
 
 /**
@@ -254,6 +258,21 @@ gs_plugin_alloc_data (GsPlugin *plugin, gsize sz)
 	return priv->data;
 }
 
+/**
+ * gs_plugin_clear_data:
+ * @plugin: a #GsPlugin
+ *
+ * Clears and resets the private data. Only run this from the self tests.
+ **/
+void
+gs_plugin_clear_data (GsPlugin *plugin)
+{
+	GsPluginPrivate *priv = gs_plugin_get_instance_private (plugin);
+	if (priv->data == NULL)
+		return;
+	g_clear_pointer (&priv->data, g_free);
+}
+
 /**
  * gs_plugin_action_start:
  * @plugin: a #GsPlugin
@@ -345,6 +364,8 @@ gs_plugin_get_symbol (GsPlugin *plugin, const gchar *function_name)
 	gpointer func = NULL;
 	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&priv->vfuncs_mutex);
 
+	g_return_val_if_fail (function_name != NULL, NULL);
+
 	/* disabled plugins shouldn't be checked */
 	if (!priv->enabled)
 		return NULL;
@@ -442,7 +463,7 @@ gs_plugin_set_appstream_id (GsPlugin *plugin, const gchar *appstream_id)
 {
 	GsPluginPrivate *priv = gs_plugin_get_instance_private (plugin);
 	g_free (priv->appstream_id);
-	appstream_id = g_strdup (appstream_id);
+	priv->appstream_id = g_strdup (appstream_id);
 }
 
 /**
@@ -917,7 +938,8 @@ gs_plugin_status_update_cb (gpointer user_data)
 		       signals[SIGNAL_STATUS_CHANGED], 0,
 		       helper->app,
 		       helper->status);
-	g_object_unref (helper->app);
+	if (helper->app != NULL)
+		g_object_unref (helper->app);
 	g_slice_free (GsPluginStatusHelper, helper);
 	return FALSE;
 }
@@ -939,9 +961,7 @@ gs_plugin_status_update (GsPlugin *plugin, GsApp *app, GsPluginStatus status)
 	helper = g_slice_new0 (GsPluginStatusHelper);
 	helper->plugin = plugin;
 	helper->status = status;
-	if (app == NULL)
-		helper->app = gs_app_new (NULL);
-	else
+	if (app != NULL)
 		helper->app = g_object_ref (app);
 	g_idle_add (gs_plugin_status_update_cb, helper);
 }
@@ -1134,6 +1154,24 @@ gs_plugin_download_data (GsPlugin *plugin,
 	g_return_val_if_fail (uri != NULL, NULL);
 	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
+	/* local */
+	if (g_str_has_prefix (uri, "file://")) {
+		gsize length = 0;
+		g_autofree gchar *contents = NULL;
+		g_autoptr(GError) error_local = NULL;
+		g_debug ("copying %s from plugin %s", uri, priv->name);
+		if (!g_file_get_contents (uri + 7, &contents, &length, &error_local)) {
+			g_set_error (error,
+				     GS_PLUGIN_ERROR,
+				     GS_PLUGIN_ERROR_DOWNLOAD_FAILED,
+				     "failed to copy %s: %s",
+				     uri, error_local->message);
+			return NULL;
+		}
+		return g_bytes_new (contents, length);
+	}
+
+	/* remote */
 	g_debug ("downloading %s from plugin %s", uri, priv->name);
 	msg = soup_message_new (SOUP_METHOD_GET, uri);
 	if (app != NULL) {
@@ -1197,6 +1235,31 @@ gs_plugin_download_file (GsPlugin *plugin,
 	g_return_val_if_fail (filename != NULL, FALSE);
 	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
+	/* local */
+	if (g_str_has_prefix (uri, "file://")) {
+		gsize length = 0;
+		g_autofree gchar *contents = NULL;
+		g_debug ("copying %s from plugin %s", uri, priv->name);
+		if (!g_file_get_contents (uri + 7, &contents, &length, &error_local)) {
+			g_set_error (error,
+				     GS_PLUGIN_ERROR,
+				     GS_PLUGIN_ERROR_DOWNLOAD_FAILED,
+				     "failed to copy %s: %s",
+				     uri, error_local->message);
+			return FALSE;
+		}
+		if (!g_file_set_contents (filename, contents, length, &error_local)) {
+			g_set_error (error,
+				     GS_PLUGIN_ERROR,
+				     GS_PLUGIN_ERROR_WRITE_FAILED,
+				     "Failed to save file: %s",
+				     error_local->message);
+			return FALSE;
+		}
+		return TRUE;
+	}
+
+	/* remote */
 	g_debug ("downloading %s to %s from plugin %s", uri, filename, priv->name);
 	msg = soup_message_new (SOUP_METHOD_GET, uri);
 	if (app != NULL) {
