@@ -88,6 +88,15 @@ gs_update_list_remove_all (GsUpdateList *update_list)
 	gs_container_remove_all (GTK_CONTAINER (update_list));
 }
 
+static void
+gs_update_list_app_state_notify_cb (GsApp *app, GParamSpec *pspec, gpointer user_data)
+{
+	if (gs_app_get_state (app) == AS_APP_STATE_INSTALLED) {
+		GsAppRow *app_row = GS_APP_ROW (user_data);
+		gs_app_row_unreveal (app_row);
+	}
+}
+
 void
 gs_update_list_add_app (GsUpdateList *update_list, GsApp *app)
 {
@@ -110,6 +119,9 @@ gs_update_list_add_app (GsUpdateList *update_list, GsApp *app)
 				    priv->sizegroup_image,
 				    priv->sizegroup_name,
 				    priv->sizegroup_button);
+	g_signal_connect (app, "notify::state",
+			  G_CALLBACK (gs_update_list_app_state_notify_cb),
+			  app_row);
 	gtk_widget_show (app_row);
 }
 
@@ -176,6 +188,7 @@ static void
 gs_update_list_update_offline_firmware_cb (GtkButton *button,
 					   GsUpdateList *update_list)
 {
+	gtk_widget_set_sensitive (GTK_WIDGET (button), FALSE);
 	gs_update_list_emit_clicked_for_section (update_list,
 						 GS_UPDATE_LIST_SECTION_OFFLINE_FIRMWARE);
 }
@@ -183,6 +196,7 @@ gs_update_list_update_offline_firmware_cb (GtkButton *button,
 static void
 gs_update_list_update_offline_cb (GtkButton *button, GsUpdateList *update_list)
 {
+	gtk_widget_set_sensitive (GTK_WIDGET (button), FALSE);
 	gs_update_list_emit_clicked_for_section (update_list,
 						 GS_UPDATE_LIST_SECTION_OFFLINE);
 }
@@ -190,6 +204,7 @@ gs_update_list_update_offline_cb (GtkButton *button, GsUpdateList *update_list)
 static void
 gs_update_list_update_online_cb (GtkButton *button, GsUpdateList *update_list)
 {
+	gtk_widget_set_sensitive (GTK_WIDGET (button), FALSE);
 	gs_update_list_emit_clicked_for_section (update_list,
 						 GS_UPDATE_LIST_SECTION_ONLINE);
 }
