@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2013 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2013-2016 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -23,7 +23,9 @@
 #define __GS_UPDATE_MONITOR_H
 
 #include <glib-object.h>
+
 #include "gs-application.h"
+#include "gs-shell.h"
 
 G_BEGIN_DECLS
 
@@ -31,7 +33,9 @@ G_BEGIN_DECLS
 
 G_DECLARE_FINAL_TYPE (GsUpdateMonitor, gs_update_monitor, GS, UPDATE_MONITOR, GObject)
 
-GsUpdateMonitor	*gs_update_monitor_new		(GsApplication *app);
+GsUpdateMonitor	*gs_update_monitor_new			(GsApplication	*app);
+void		 gs_update_monitor_show_error		(GsUpdateMonitor *monitor,
+							 GsShell	*shell);
 
 G_END_DECLS
 
