@@ -37,12 +37,15 @@
 
 struct _GsFlatpak {
 	GObject			 parent_instance;
+	GsFlatpakFlags		 flags;
 	FlatpakInstallation	*installation;
 	GHashTable		*broken_remotes;
 	GFileMonitor		*monitor;
 	AsAppScope		 scope;
 	GsPlugin		*plugin;
 	AsStore			*store;
+	gchar			*id;
+	guint			 changed_id;
 };
 
 G_DEFINE_TYPE (GsFlatpak, gs_flatpak, G_TYPE_OBJECT)
@@ -69,6 +72,10 @@ gs_plugin_flatpak_error_convert (GError **perror)
 	if (gs_utils_error_convert_gdbus (perror))
 		return;
 
+	/* this are allowed for network ops */
+	if (gs_utils_error_convert_gresolver (perror))
+		return;
+
 	/* custom to this plugin */
 	if (error->domain == FLATPAK_ERROR) {
 		switch (error->code) {
@@ -110,6 +117,7 @@ gs_flatpak_create_app (GsFlatpak *self, FlatpakRef *xref)
 	app = gs_app_new (id);
 	gs_app_set_bundle_kind (app, AS_BUNDLE_KIND_FLATPAK);
 	gs_app_set_branch (app, flatpak_ref_get_branch (xref));
+	gs_app_set_flatpak_object_id (app, gs_flatpak_get_id (self));
 	if (flatpak_installation_get_is_user (self->installation)) {
 		gs_app_set_scope (app, AS_APP_SCOPE_USER);
 	} else {
@@ -354,8 +362,17 @@ gs_flatpak_rescan_installed (GsFlatpak *self,
 		/* add */
 		as_app_set_state (app, AS_APP_STATE_INSTALLED);
 		as_app_set_scope (app, self->scope);
+#if AS_CHECK_VERSION(0,6,9)
+{
+		g_autoptr(AsFormat) format = as_format_new ();
+		as_format_set_kind (format, AS_FORMAT_KIND_DESKTOP);
+		as_format_set_filename (format, fn_desktop);
+		as_app_add_format (app, format);
+}
+#else
 		as_app_set_source_kind (app, AS_APP_SOURCE_KIND_DESKTOP);
 		as_app_set_source_file (app, fn_desktop);
+#endif
 		as_app_set_icon_path (app, path_exports);
 		as_app_add_keyword (app, NULL, "flatpak");
 		as_store_add_app (self->store, app);
@@ -440,8 +457,9 @@ gs_flatpak_setup (GsFlatpak *self, GCancellable *cancellable, GError **error)
 		gs_plugin_flatpak_error_convert (error);
 		return FALSE;
 	}
-	g_signal_connect (self->monitor, "changed",
-			  G_CALLBACK (gs_plugin_flatpak_changed_cb), self);
+	self->changed_id =
+		g_signal_connect (self->monitor, "changed",
+				  G_CALLBACK (gs_plugin_flatpak_changed_cb), self);
 
 	/* ensure the legacy AppStream symlink cache is deleted */
 	if (!gs_flatpak_symlinks_cleanup (self->installation, cancellable, error))
@@ -872,7 +890,7 @@ gs_flatpak_app_install_source (GsFlatpak *self, GsApp *app,
 
 	/* decode GPG key if set */
 	gpg_key = gs_app_get_metadata_item (app, "flatpak::gpg-key");
-	if (gpg_key != NULL) {
+	if (gpg_key != NULL && g_strcmp0 (gpg_key, "FOOBAR==") != 0) {
 		gsize data_len = 0;
 		g_autofree guchar *data = NULL;
 		g_autoptr(GBytes) bytes = NULL;
@@ -896,13 +914,16 @@ gs_flatpak_app_install_source (GsFlatpak *self, GsApp *app,
 						 cancellable,
 						 error)) {
 		gs_plugin_flatpak_error_convert (error);
+		g_prefix_error (error, "cannot modify remote: ");
 		gs_app_set_state_recover (app);
 		return FALSE;
 	}
 
 	/* refresh the AppStream data manually */
-	if (!gs_flatpak_add_apps_from_xremote (self, xremote, cancellable, error))
+	if (!gs_flatpak_add_apps_from_xremote (self, xremote, cancellable, error)) {
+		g_prefix_error (error, "cannot refresh remote AppStream: ");
 		return FALSE;
+	}
 
 	/* success */
 	gs_app_set_state (app, AS_APP_STATE_INSTALLED);
@@ -984,8 +1005,8 @@ gs_flatpak_add_updates (GsFlatpak *self, GsAppList *list,
 			GError **error)
 {
 	g_autoptr(GPtrArray) xrefs = NULL;
-	g_autoptr(GsAppList) list_new = gs_app_list_new ();
-	g_autoptr(GsAppList) list_tmp = gs_app_list_new ();
+	g_autoptr(GsAppList) list_new = NULL;
+	g_autoptr(GsAppList) list_tmp = NULL;
 
 	/* get all the installed apps (no network I/O) */
 	xrefs = flatpak_installation_list_installed_refs (self->installation,
@@ -997,6 +1018,7 @@ gs_flatpak_add_updates (GsFlatpak *self, GsAppList *list,
 	}
 
 	/* look at each installed xref */
+	list_tmp = gs_app_list_new ();
 	for (guint i = 0; i < xrefs->len; i++) {
 		FlatpakInstalledRef *xref = g_ptr_array_index (xrefs, i);
 		const gchar *commit;
@@ -1028,6 +1050,9 @@ gs_flatpak_add_updates (GsFlatpak *self, GsAppList *list,
 			continue;
 		}
 		gs_app_set_state (app, AS_APP_STATE_UPDATABLE_LIVE);
+		gs_app_set_update_details (app, NULL);
+		gs_app_set_update_version (app, NULL);
+		gs_app_set_update_urgency (app, AS_URGENCY_KIND_UNKNOWN);
 		gs_app_set_size_download (app, 0);
 		gs_app_list_add (list_tmp, app);
 	}
@@ -1321,11 +1346,11 @@ gs_app_get_flatpak_kind (GsApp *app)
 }
 
 static gboolean
-refine_origin_from_installation (GsFlatpak *self,
-				 FlatpakInstallation *installation,
-				 GsApp *app,
-				 GCancellable *cancellable,
-				 GError **error)
+gs_flatpak_refine_origin_from_installation (GsFlatpak *self,
+					    FlatpakInstallation *installation,
+					    GsApp *app,
+					    GCancellable *cancellable,
+					    GError **error)
 {
 	guint i;
 	g_autoptr(GPtrArray) xremotes = NULL;
@@ -1341,6 +1366,7 @@ refine_origin_from_installation (GsFlatpak *self,
 		const gchar *remote_name;
 		FlatpakRemote *xremote = g_ptr_array_index (xremotes, i);
 		g_autoptr(FlatpakRemoteRef) xref = NULL;
+		g_autoptr(GError) error_local = NULL;
 
 		/* not enabled */
 		if (flatpak_remote_get_disabled (xremote))
@@ -1356,22 +1382,18 @@ refine_origin_from_installation (GsFlatpak *self,
 								   gs_app_get_flatpak_arch (app),
 								   gs_app_get_flatpak_branch (app),
 								   cancellable,
-								   NULL);
+								   &error_local);
 		if (xref != NULL) {
 			g_debug ("found remote %s", remote_name);
 			gs_app_set_origin (app, remote_name);
 			return TRUE;
 		}
+		g_debug ("failed to find remote %s: %s",
+			 remote_name, error_local->message);
 	}
-	g_set_error (error,
-		     GS_PLUGIN_ERROR,
-		     GS_PLUGIN_ERROR_NOT_SUPPORTED,
-		     "Not found %s/%s/%s",
-		     gs_app_get_flatpak_name (app),
-		     gs_app_get_flatpak_arch (app),
-		     gs_app_get_flatpak_branch (app));
 
-	return FALSE;
+	/* not found */
+	return TRUE;
 }
 
 static FlatpakInstallation *
@@ -1399,7 +1421,6 @@ gs_plugin_refine_item_origin (GsFlatpak *self,
 {
 	g_autoptr(AsProfileTask) ptask = NULL;
 	g_autoptr(GError) local_error = NULL;
-	gboolean ignore_error = FALSE;
 
 	/* already set */
 	if (gs_app_get_origin (app) != NULL)
@@ -1423,36 +1444,39 @@ gs_plugin_refine_item_origin (GsFlatpak *self,
 		 gs_app_get_flatpak_branch (app));
 
 	/* first check the plugin's own flatpak installation */
-	if (refine_origin_from_installation (self, self->installation, app,
-					     cancellable, &local_error)) {
-		return TRUE;
+	if (!gs_flatpak_refine_origin_from_installation (self,
+							 self->installation,
+							 app,
+							 cancellable,
+							 error)) {
+		g_prefix_error (error, "failed to refine origin from self: ");
+		return FALSE;
 	}
 
-	ignore_error = g_error_matches (local_error, GS_PLUGIN_ERROR,
-					GS_PLUGIN_ERROR_NOT_SUPPORTED);
-
 	/* check the system installation if we're on a user one */
-	if (ignore_error &&
+	if (gs_app_get_scope (app) == AS_APP_SCOPE_USER &&
 	    gs_app_get_flatpak_kind (app) == FLATPAK_REF_KIND_RUNTIME) {
 		g_autoptr(FlatpakInstallation) installation =
 			gs_flatpak_get_installation_counterpart (self,
 								 cancellable,
 								 error);
 
-		if (installation == NULL)
+		if (installation == NULL) {
+			g_prefix_error (error, "failed to get counterpart: ");
+			return FALSE;
+		}
+		if (!gs_flatpak_refine_origin_from_installation (self,
+								 installation,
+								 app,
+								 cancellable,
+								 error)) {
+			g_prefix_error (error,
+					"failed to refine origin from counterpart: ");
 			return FALSE;
-
-		if (refine_origin_from_installation (self, installation, app,
-						     cancellable, error)) {
-			return TRUE;
 		}
-	} else {
-		g_propagate_error (error, local_error);
-		/* safely handle the autoptr */
-		local_error = NULL;
 	}
 
-	return FALSE;
+	return TRUE;
 }
 
 static gboolean
@@ -1547,20 +1571,21 @@ gs_plugin_refine_item_state (GsFlatpak *self,
 	 * available system-wide then mark it installed, and vice-versa */
 	if (gs_app_get_flatpak_kind (app) == FLATPAK_REF_KIND_RUNTIME &&
 	    gs_app_get_state (app) == AS_APP_STATE_UNKNOWN) {
+		g_autoptr(GPtrArray) xrefs2 = NULL;
 		g_autoptr(FlatpakInstallation) installation =
 			gs_flatpak_get_installation_counterpart (self,
 								 cancellable,
 								 error);
 		if (installation == NULL)
 			return FALSE;
-		xrefs = flatpak_installation_list_installed_refs (installation,
-								  cancellable, error);
-		if (xrefs == NULL) {
+		xrefs2 = flatpak_installation_list_installed_refs (installation,
+								   cancellable, error);
+		if (xrefs2 == NULL) {
 			gs_plugin_flatpak_error_convert (error);
 			return FALSE;
 		}
-		for (i = 0; i < xrefs->len; i++) {
-			FlatpakInstalledRef *xref = g_ptr_array_index (xrefs, i);
+		for (i = 0; i < xrefs2->len; i++) {
+			FlatpakInstalledRef *xref = g_ptr_array_index (xrefs2, i);
 			if (!gs_flatpak_app_matches_xref (self, app, FLATPAK_REF(xref)))
 				continue;
 			gs_app_set_state (app, AS_APP_STATE_INSTALLED);
@@ -1587,11 +1612,11 @@ gs_plugin_refine_item_state (GsFlatpak *self,
 				gs_app_set_state (app, AS_APP_STATE_AVAILABLE);
 			}
 		} else {
-			g_warning ("failed to find flatpak %s remote %s for %s",
-				   flatpak_installation_get_is_user (self->installation) ? "user" : "system",
-				   gs_app_get_origin (app),
-				   gs_app_get_unique_id (app));
-			g_warning ("%s", gs_app_to_string (app));
+			gs_app_set_state (app, AS_APP_STATE_UNKNOWN);
+			g_debug ("failed to find %s remote %s for %s",
+				 self->id,
+				 gs_app_get_origin (app),
+				 gs_app_get_unique_id (app));
 		}
 	}
 
@@ -1778,6 +1803,8 @@ gs_plugin_refine_item_metadata (GsFlatpak *self,
 	} else {
 		data = gs_flatpak_fetch_remote_metadata (self, app, cancellable,
 							 error);
+		if (data == NULL)
+			return FALSE;
 		str = g_bytes_get_data (data, &len);
 	}
 
@@ -1916,6 +1943,27 @@ gs_plugin_refine_item_size (GsFlatpak *self,
 	return TRUE;
 }
 
+static void
+gs_flatpak_refine_appstream_release (AsApp *item, GsApp *app)
+{
+	AsRelease *rel = as_app_get_release_default (item);
+	if (rel == NULL)
+		return;
+	if (as_release_get_version (rel) == NULL)
+		return;
+	switch (gs_app_get_state (app)) {
+	case AS_APP_STATE_INSTALLED:
+	case AS_APP_STATE_AVAILABLE:
+	case AS_APP_STATE_AVAILABLE_LOCAL:
+		gs_app_set_version (app, as_release_get_version (rel));
+		break;
+	default:
+		g_debug ("%s is not installed, so ignoring version of %s",
+			 as_app_get_id (item), as_release_get_version (rel));
+		break;
+	}
+}
+
 static gboolean
 gs_flatpak_refine_appstream (GsFlatpak *self, GsApp *app, GError **error)
 {
@@ -1934,9 +1982,26 @@ gs_flatpak_refine_appstream (GsFlatpak *self, GsApp *app, GError **error)
 	item = as_store_get_app_by_unique_id (self->store,
 					      unique_id,
 					      AS_STORE_SEARCH_FLAG_USE_WILDCARDS);
-	if (item == NULL)
+	if (item == NULL) {
+		g_autoptr(GPtrArray) apps = NULL;
+		apps = as_store_get_apps_by_id (self->store, gs_app_get_id (app));
+		if (apps->len > 0) {
+			g_debug ("potential matches for %s:", unique_id);
+			for (guint i = 0; i < apps->len; i++) {
+				AsApp *app_tmp = g_ptr_array_index (apps, i);
+				g_debug ("- %s", as_app_get_unique_id (app_tmp));
+			}
+		}
 		return TRUE;
-	return gs_appstream_refine_app (self->plugin, app, item, error);
+	}
+
+	if (!gs_appstream_refine_app (self->plugin, app, item, error))
+		return FALSE;
+
+	/* use the default release as the version number */
+	gs_flatpak_refine_appstream_release (item, app);
+
+	return TRUE;
 }
 
 gboolean
@@ -2151,6 +2216,9 @@ gs_flatpak_app_remove (GsFlatpak *self,
 		       GCancellable *cancellable,
 		       GError **error)
 {
+	g_autoptr(FlatpakRemote) xremote = NULL;
+	g_autofree gchar *remote_name = NULL;
+
 	/* refine to get basics */
 	if (!gs_flatpak_refine_app (self, app,
 				    GS_PLUGIN_REFINE_FLAGS_DEFAULT,
@@ -2179,6 +2247,26 @@ gs_flatpak_app_remove (GsFlatpak *self,
 		return FALSE;
 	}
 
+	/* did app also install a noenumerate=True remote */
+	remote_name = g_strdup_printf ("%s-origin", gs_app_get_flatpak_name (app));
+	xremote = flatpak_installation_get_remote_by_name (self->installation,
+							   remote_name,
+							   cancellable,
+							   NULL);
+	if (xremote != NULL) {
+		g_debug ("removing enumerate=true %s remote", remote_name);
+		if (!flatpak_installation_remove_remote (self->installation,
+							 remote_name,
+							 cancellable,
+							 error)) {
+			gs_plugin_flatpak_error_convert (error);
+			gs_app_set_state_recover (app);
+			return FALSE;
+		}
+		if (!gs_flatpak_rescan_appstream_store (self, cancellable, error))
+			return FALSE;
+	}
+
 	/* state is not known: we don't know if we can re-install this app */
 	gs_app_set_state (app, AS_APP_STATE_UNKNOWN);
 
@@ -2208,7 +2296,6 @@ install_runtime_for_app (GsFlatpak *self,
 		data = gs_flatpak_fetch_remote_metadata (self, app, cancellable, error);
 		if (data == NULL) {
 			gs_utils_error_add_unique_id (error, app);
-			gs_app_set_state_recover (app);
 			return FALSE;
 		}
 
@@ -2216,6 +2303,8 @@ install_runtime_for_app (GsFlatpak *self,
 		runtime_new = gs_flatpak_create_runtime_from_metadata (self, app,
 								       str, len,
 								       error);
+		if (runtime_new == NULL)
+			return FALSE;
 		gs_app_set_update_runtime (app, runtime_new);
 	}
 
@@ -2227,17 +2316,14 @@ install_runtime_for_app (GsFlatpak *self,
 	/* the runtime could come from a different remote to the app */
 	if (!gs_refine_item_metadata (self, runtime, cancellable, error)) {
 		gs_utils_error_add_unique_id (error, runtime);
-		gs_app_set_state_recover (app);
 		return FALSE;
 	}
 	if (!gs_plugin_refine_item_origin (self, runtime, cancellable, error)) {
 		gs_utils_error_add_unique_id (error, runtime);
-		gs_app_set_state_recover (app);
 		return FALSE;
 	}
 	if (!gs_plugin_refine_item_state (self, runtime, cancellable, error)) {
 		gs_utils_error_add_unique_id (error, runtime);
-		gs_app_set_state_recover (app);
 		return FALSE;
 	}
 	if (gs_app_get_state (runtime) == AS_APP_STATE_UNKNOWN) {
@@ -2247,7 +2333,6 @@ install_runtime_for_app (GsFlatpak *self,
 			     "Failed to find runtime %s",
 			     gs_app_get_source_default (runtime));
 		gs_utils_error_add_unique_id (error, runtime);
-		gs_app_set_state_recover (app);
 		return FALSE;
 	}
 
@@ -2269,8 +2354,8 @@ install_runtime_for_app (GsFlatpak *self,
 						     gs_flatpak_progress_cb, app,
 						     cancellable, error);
 		if (xref == NULL) {
+			gs_plugin_flatpak_error_convert (error);
 			gs_app_set_state_recover (runtime);
-			gs_app_set_state_recover (app);
 			return FALSE;
 		}
 		gs_app_set_state (runtime, AS_APP_STATE_INSTALLED);
@@ -2284,6 +2369,120 @@ install_runtime_for_app (GsFlatpak *self,
 	return TRUE;
 }
 
+static GsApp *
+gs_flatpak_create_app_from_repo_file (GsFlatpak *self,
+				      GFile *file,
+				      GCancellable *cancellable,
+				      GError **error)
+{
+	gchar *tmp;
+	g_autofree gchar *filename = NULL;
+	g_autofree gchar *repo_comment = NULL;
+	g_autofree gchar *repo_default_branch = NULL;
+	g_autofree gchar *repo_description = NULL;
+	g_autofree gchar *repo_gpgkey = NULL;
+	g_autofree gchar *repo_homepage = NULL;
+	g_autofree gchar *repo_icon = NULL;
+	g_autofree gchar *repo_id = NULL;
+	g_autofree gchar *repo_title = NULL;
+	g_autofree gchar *repo_url = NULL;
+	g_autoptr(GError) error_local = NULL;
+	g_autoptr(GKeyFile) kf = NULL;
+	g_autoptr(GsApp) app = NULL;
+
+	/* read the file */
+	kf = g_key_file_new ();
+	filename = g_file_get_path (file);
+	if (!g_key_file_load_from_file (kf, filename,
+					G_KEY_FILE_NONE,
+					&error_local)) {
+		g_set_error (error,
+			     GS_PLUGIN_ERROR,
+			     GS_PLUGIN_ERROR_NOT_SUPPORTED,
+			     "failed to load flatpakrepo: %s",
+			     error_local->message);
+		return NULL;
+	}
+
+	/* get the ID from the basename */
+	repo_id = g_file_get_basename (file);
+	tmp = g_strrstr (repo_id, ".");
+	if (tmp != NULL)
+		*tmp = '\0';
+
+	/* create source */
+	repo_title = g_key_file_get_string (kf, "Flatpak Repo", "Title", NULL);
+	repo_url = g_key_file_get_string (kf, "Flatpak Repo", "Url", NULL);
+	repo_gpgkey = g_key_file_get_string (kf, "Flatpak Repo", "GPGKey", NULL);
+	if (repo_title == NULL || repo_url == NULL || repo_gpgkey == NULL ||
+	    repo_title[0] == '\0' || repo_url[0] == '\0' || repo_gpgkey[0] == '\0') {
+		g_set_error_literal (error,
+				     GS_PLUGIN_ERROR,
+				     GS_PLUGIN_ERROR_NOT_SUPPORTED,
+				     "not enough data in file, "
+				     "expected Title, Url, GPGKey");
+		return NULL;
+	}
+
+	/* check version */
+	if (g_key_file_has_key (kf, "Flatpak Repo", "Version", NULL)) {
+		guint64 ver = g_key_file_get_uint64 (kf, "Flatpak Repo", "Version", NULL);
+		if (ver != 1) {
+			g_set_error (error,
+				     GS_PLUGIN_ERROR,
+				     GS_PLUGIN_ERROR_NOT_SUPPORTED,
+				     "unsupported version %" G_GUINT64_FORMAT, ver);
+			return NULL;
+		}
+	}
+
+	/* user specified a URL */
+	if (g_str_has_prefix (repo_gpgkey, "http://") ||
+	    g_str_has_prefix (repo_gpgkey, "https://")) {
+		g_set_error_literal (error,
+				     GS_PLUGIN_ERROR,
+				     GS_PLUGIN_ERROR_NOT_SUPPORTED,
+				     "Base64 encoded GPGKey required, not URL");
+		return NULL;
+	}
+
+	/* create source */
+	app = gs_app_new (repo_id);
+	gs_app_set_flatpak_file_type (app, "flatpakrepo");
+	gs_app_set_kind (app, AS_APP_KIND_SOURCE);
+	gs_app_set_state (app, AS_APP_STATE_AVAILABLE);
+	gs_app_add_quirk (app, AS_APP_QUIRK_NOT_LAUNCHABLE);
+	gs_app_set_name (app, GS_APP_QUALITY_NORMAL, repo_title);
+	gs_app_set_metadata (app, "flatpak::gpg-key", repo_gpgkey);
+	gs_app_set_metadata (app, "flatpak::url", repo_url);
+	gs_app_set_origin_hostname (app, repo_url);
+	gs_app_set_management_plugin (app, gs_plugin_get_name (self->plugin));
+
+	/* optional data */
+	repo_homepage = g_key_file_get_string (kf, "Flatpak Repo", "Homepage", NULL);
+	if (repo_homepage != NULL)
+		gs_app_set_url (app, AS_URL_KIND_HOMEPAGE, repo_homepage);
+	repo_comment = g_key_file_get_string (kf, "Flatpak Repo", "Comment", NULL);
+	if (repo_comment != NULL)
+		gs_app_set_summary (app, GS_APP_QUALITY_NORMAL, repo_comment);
+	repo_description = g_key_file_get_string (kf, "Flatpak Repo", "Description", NULL);
+	if (repo_description != NULL)
+		gs_app_set_description (app, GS_APP_QUALITY_NORMAL, repo_description);
+	repo_default_branch = g_key_file_get_string (kf, "Flatpak Repo", "DefaultBranch", NULL);
+	if (repo_default_branch != NULL)
+		gs_app_set_branch (app, repo_default_branch);
+	repo_icon = g_key_file_get_string (kf, "Flatpak Repo", "Icon", NULL);
+	if (repo_icon != NULL) {
+		g_autoptr(AsIcon) ic = as_icon_new ();
+		as_icon_set_kind (ic, AS_ICON_KIND_REMOTE);
+		as_icon_set_url (ic, repo_icon);
+		gs_app_add_icon (app, ic);
+	}
+
+	/* success */
+	return g_steal_pointer (&app);
+}
+
 gboolean
 gs_flatpak_app_install (GsFlatpak *self,
 			GsApp *app,
@@ -2293,8 +2492,9 @@ gs_flatpak_app_install (GsFlatpak *self,
 	g_autoptr(FlatpakInstalledRef) xref = NULL;
 
 	/* ensure we have metadata and state */
-	if (!gs_flatpak_refine_app (self, app, 0, cancellable,
-				    error))
+	if (!gs_flatpak_refine_app (self, app,
+				    GS_PLUGIN_REFINE_FLAGS_REQUIRE_RUNTIME,
+				    cancellable, error))
 		return FALSE;
 
 	/* install */
@@ -2308,10 +2508,127 @@ gs_flatpak_app_install (GsFlatpak *self,
 						      error);
 	}
 
+	/* flatpakref has to be done in two phases */
+	if (g_strcmp0 (gs_app_get_flatpak_file_type (app), "flatpakref") == 0) {
+		GsApp *runtime;
+		g_autoptr(FlatpakRemoteRef) xref2 = NULL;
+		gsize len = 0;
+		g_autofree gchar *contents = NULL;
+		g_autoptr(GBytes) data = NULL;
+		if (gs_app_get_local_file (app) == NULL) {
+			g_set_error (error,
+				     GS_PLUGIN_ERROR,
+				     GS_PLUGIN_ERROR_NOT_SUPPORTED,
+				     "no local file set for flatpakref %s",
+				     gs_app_get_unique_id (app));
+			gs_app_set_state_recover (app);
+			return FALSE;
+		}
+		g_debug ("installing flatpakref %s", gs_app_get_unique_id (app));
+		if (!g_file_load_contents (gs_app_get_local_file (app),
+					   cancellable, &contents, &len,
+					   NULL, error)) {
+			gs_utils_error_convert_gio (error);
+			gs_app_set_state_recover (app);
+			return FALSE;
+		}
+		data = g_bytes_new (contents, len);
+		xref2 = flatpak_installation_install_ref_file (self->installation,
+							      data,
+							      cancellable,
+							      error);
+		if (xref2 == NULL) {
+			gs_plugin_flatpak_error_convert (error);
+			gs_app_set_state_recover (app);
+			return FALSE;
+		}
+
+		/* we have a missing remote and a RuntimeRef */
+		runtime = gs_app_get_runtime (app);
+		if (runtime != NULL && gs_app_get_state (runtime) == AS_APP_STATE_UNKNOWN) {
+			g_autofree gchar *cache_basename = NULL;
+			g_autofree gchar *cache_fn = NULL;
+			g_autoptr(GFile) file = NULL;
+			g_autoptr(GsApp) app_src = NULL;
+			const gchar *tmp = gs_app_get_metadata_item (app, "flatpak::runtime-repo");
+			if (tmp == NULL) {
+				g_set_error (error,
+					     GS_PLUGIN_ERROR,
+					     GS_PLUGIN_ERROR_NOT_SUPPORTED,
+					     "no runtime available for %s",
+					     gs_app_get_unique_id (app));
+				gs_utils_error_add_unique_id (error, runtime);
+				return FALSE;
+			}
+			g_debug ("runtime not available, so using %s", tmp);
+
+			/* download file */
+			cache_basename = g_path_get_basename (tmp);
+			cache_fn = gs_utils_get_cache_filename ("flatpak",
+								cache_basename,
+								GS_UTILS_CACHE_FLAG_WRITEABLE,
+								error);
+			if (cache_fn == NULL)
+				return FALSE;
+			if (!gs_plugin_download_file (self->plugin,
+						      runtime,
+						      tmp,
+						      cache_fn,
+						      cancellable,
+						      error))
+				return FALSE;
+
+			/* get GsApp for local file */
+			file = g_file_new_for_path (cache_fn);
+			app_src = gs_flatpak_create_app_from_repo_file (self,
+									file,
+									cancellable,
+									error);
+			if (app_src == NULL) {
+				g_prefix_error (error,
+						"cannot create source from %s: ",
+						cache_fn);
+				return FALSE;
+			}
+
+			/* install the flatpakrepo */
+			if (!gs_flatpak_app_install_source (self,
+							    app_src,
+							    cancellable,
+							    error)) {
+				g_prefix_error (error,
+						"cannot install source from %s: ",
+						cache_fn);
+				return FALSE;
+			}
+
+			/* get the new state */
+			if (!gs_plugin_refine_item_state (self, runtime, cancellable, error)) {
+				g_prefix_error (error,
+						"cannot refine runtime using %s: ",
+						cache_fn);
+				return FALSE;
+			}
+
+			/* still not found */
+			if (gs_app_get_state (runtime) == AS_APP_STATE_UNKNOWN) {
+				g_set_error (error,
+					     GS_PLUGIN_ERROR,
+					     GS_PLUGIN_ERROR_NOT_SUPPORTED,
+					     "no runtime available for %s",
+					     gs_app_get_unique_id (app));
+				gs_utils_error_add_unique_id (error, runtime);
+				return FALSE;
+			}
+		}
+	}
+
 	/* install required runtime if not already installed */
 	if (gs_app_get_kind (app) == AS_APP_KIND_DESKTOP &&
-	    !install_runtime_for_app (self, app, cancellable, error))
+	    !install_runtime_for_app (self, app, cancellable, error)) {
+		gs_app_set_state_recover (app);
 		return FALSE;
+	}
 
 	if (g_strcmp0 (gs_app_get_flatpak_file_type (app), "flatpak") == 0) {
 		if (gs_app_get_local_file (app) == NULL) {
@@ -2356,6 +2673,11 @@ gs_flatpak_app_install (GsFlatpak *self,
 
 	/* state is known */
 	gs_app_set_state (app, AS_APP_STATE_INSTALLED);
+
+	/* set new version */
+	if (!gs_flatpak_refine_appstream (self, app, error))
+		return FALSE;
+
 	return TRUE;
 }
 
@@ -2372,8 +2694,10 @@ gs_flatpak_update_app (GsFlatpak *self,
 
 	/* install required runtime if not already installed */
 	if (gs_app_get_kind (app) == AS_APP_KIND_DESKTOP &&
-	    !install_runtime_for_app (self, app, cancellable, error))
+	    !install_runtime_for_app (self, app, cancellable, error)) {
+		gs_app_set_state_recover (app);
 		return FALSE;
+	}
 
 	xref = flatpak_installation_update (self->installation,
 					    FLATPAK_UPDATE_FLAGS_NONE,
@@ -2394,6 +2718,14 @@ gs_flatpak_update_app (GsFlatpak *self,
 
 	/* state is known */
 	gs_app_set_state (app, AS_APP_STATE_INSTALLED);
+	gs_app_set_update_version (app, NULL);
+	gs_app_set_update_details (app, NULL);
+	gs_app_set_update_urgency (app, AS_URGENCY_KIND_UNKNOWN);
+
+	/* set new version */
+	if (!gs_flatpak_refine_appstream (self, app, error))
+		return FALSE;
+
 	return TRUE;
 }
 
@@ -2549,116 +2881,21 @@ gs_flatpak_file_to_app_repo (GsFlatpak *self,
 			     GCancellable *cancellable,
 			     GError **error)
 {
-	gchar *tmp;
-	g_autofree gchar *filename = NULL;
-	g_autofree gchar *repo_comment = NULL;
-	g_autofree gchar *repo_description = NULL;
-	g_autofree gchar *repo_default_branch = NULL;
-	g_autofree gchar *repo_gpgkey = NULL;
-	g_autofree gchar *repo_homepage = NULL;
-	g_autofree gchar *repo_icon = NULL;
-	g_autofree gchar *repo_title = NULL;
-	g_autofree gchar *repo_url = NULL;
-	g_autofree gchar *repo_id = NULL;
-	g_autoptr(GError) error_local = NULL;
-	g_autoptr(GKeyFile) kf = NULL;
 	g_autoptr(GsApp) app = NULL;
 	g_autoptr(FlatpakRemote) xremote = NULL;
 
-	/* read the file */
-	kf = g_key_file_new ();
-	filename = g_file_get_path (file);
-	if (!g_key_file_load_from_file (kf, filename,
-					G_KEY_FILE_NONE,
-					&error_local)) {
-		g_set_error (error,
-			     GS_PLUGIN_ERROR,
-			     GS_PLUGIN_ERROR_NOT_SUPPORTED,
-			     "failed to load flatpakrepo: %s",
-			     error_local->message);
+	/* create app */
+	app = gs_flatpak_create_app_from_repo_file (self, file, cancellable, error);
+	if (app == NULL)
 		return FALSE;
-	}
-
-	/* get the ID from the basename */
-	repo_id = g_file_get_basename (file);
-	tmp = g_strrstr (repo_id, ".");
-	if (tmp != NULL)
-		*tmp = '\0';
-
-	/* create source */
-	repo_title = g_key_file_get_string (kf, "Flatpak Repo", "Title", NULL);
-	repo_url = g_key_file_get_string (kf, "Flatpak Repo", "Url", NULL);
-	repo_gpgkey = g_key_file_get_string (kf, "Flatpak Repo", "GPGKey", NULL);
-	if (repo_title == NULL || repo_url == NULL || repo_gpgkey == NULL ||
-	    repo_title[0] == '\0' || repo_url[0] == '\0' || repo_gpgkey[0] == '\0') {
-		g_set_error_literal (error,
-				     GS_PLUGIN_ERROR,
-				     GS_PLUGIN_ERROR_NOT_SUPPORTED,
-				     "not enough data in file, "
-				     "expected Title, Url, GPGKey");
-		return FALSE;
-	}
-
-	/* check version */
-	if (g_key_file_has_key (kf, "Flatpak Repo", "Version", NULL)) {
-		guint64 ver = g_key_file_get_uint64 (kf, "Flatpak Repo", "Version", NULL);
-		if (ver != 1) {
-			g_set_error (error,
-				     GS_PLUGIN_ERROR,
-				     GS_PLUGIN_ERROR_NOT_SUPPORTED,
-				     "unsupported version %" G_GUINT64_FORMAT, ver);
-			return FALSE;
-		}
-	}
-
-	/* user specified a URL */
-	if (g_str_has_prefix (repo_gpgkey, "http://") ||
-	    g_str_has_prefix (repo_gpgkey, "https://")) {
-		g_set_error_literal (error,
-				     GS_PLUGIN_ERROR,
-				     GS_PLUGIN_ERROR_NOT_SUPPORTED,
-				     "Base64 encoded GPGKey required, not URL");
-		return FALSE;
-	}
-
-	/* create source */
-	app = gs_app_new (repo_id);
-	gs_app_set_flatpak_file_type (app, "flatpakrepo");
-	gs_app_set_kind (app, AS_APP_KIND_SOURCE);
-	gs_app_add_quirk (app, AS_APP_QUIRK_NOT_LAUNCHABLE);
-	gs_app_set_name (app, GS_APP_QUALITY_NORMAL, repo_title);
-	gs_app_set_metadata (app, "flatpak::gpg-key", repo_gpgkey);
-	gs_app_set_metadata (app, "flatpak::url", repo_url);
-	gs_app_set_origin_hostname (app, repo_url);
-	gs_app_set_management_plugin (app, gs_plugin_get_name (self->plugin));
-
-	/* optional data */
-	repo_homepage = g_key_file_get_string (kf, "Flatpak Repo", "Homepage", NULL);
-	if (repo_homepage != NULL)
-		gs_app_set_url (app, AS_URL_KIND_HOMEPAGE, repo_homepage);
-	repo_comment = g_key_file_get_string (kf, "Flatpak Repo", "Comment", NULL);
-	if (repo_comment != NULL)
-		gs_app_set_summary (app, GS_APP_QUALITY_NORMAL, repo_comment);
-	repo_description = g_key_file_get_string (kf, "Flatpak Repo", "Description", NULL);
-	if (repo_description != NULL)
-		gs_app_set_description (app, GS_APP_QUALITY_NORMAL, repo_description);
-	repo_default_branch = g_key_file_get_string (kf, "Flatpak Repo", "DefaultBranch", NULL);
-	if (repo_default_branch != NULL)
-		gs_app_set_branch (app, repo_default_branch);
-	repo_icon = g_key_file_get_string (kf, "Flatpak Repo", "Icon", NULL);
-	if (repo_icon != NULL) {
-		g_autoptr(AsIcon) ic = as_icon_new ();
-		as_icon_set_kind (ic, AS_ICON_KIND_REMOTE);
-		as_icon_set_url (ic, repo_icon);
-		gs_app_add_icon (app, ic);
-	}
 
 	/* check to see if the repo ID already exists */
 	xremote = flatpak_installation_get_remote_by_name (self->installation,
-							   repo_id,
+							   gs_app_get_id (app),
 							   cancellable, NULL);
 	if (xremote != NULL) {
-		g_debug ("repo %s already exists", repo_id);
+		g_debug ("repo %s already exists", gs_app_get_id (app));
+		gs_app_set_state (app, AS_APP_STATE_UNKNOWN);
 		gs_app_set_state (app, AS_APP_STATE_INSTALLED);
 	} else {
 		gs_app_set_state (app, AS_APP_STATE_AVAILABLE);
@@ -2691,6 +2928,8 @@ gs_flatpak_file_to_app_ref (GsFlatpak *self,
 	g_autofree gchar *ref_homepage = NULL;
 	g_autofree gchar *ref_icon = NULL;
 	g_autofree gchar *ref_title = NULL;
+	g_autofree gchar *ref_name = NULL;
+	g_autofree gchar *ref_runtime_repo = NULL;
 
 	/* get file data */
 	if (!g_file_load_contents (file,
@@ -2722,6 +2961,41 @@ gs_flatpak_file_to_app_ref (GsFlatpak *self,
 		}
 	}
 
+	/* get name */
+	ref_name = g_key_file_get_string (kf, "Flatpak Ref", "Name", error);
+	if (ref_name == NULL) {
+		gs_utils_error_convert_gio (error);
+		return FALSE;
+	}
+
+	/* remove old version from the remote config */
+	if (self->flags & GS_FLATPAK_FLAG_IS_TEMPORARY) {
+		g_autofree gchar *remote_id_tmp = NULL;
+		g_autofree gchar *remote_name_tmp = NULL;
+		g_autoptr(FlatpakRemote) xremote_tmp = NULL;
+		remote_name_tmp = g_strdup_printf ("%s-origin", ref_name);
+		xremote_tmp = flatpak_installation_get_remote_by_name (self->installation,
+								       remote_name_tmp,
+								       cancellable,
+								       NULL);
+		if (xremote_tmp != NULL) {
+			g_debug ("removing previous remote %s", remote_name_tmp);
+			if (!flatpak_installation_remove_remote (self->installation,
+								 remote_name_tmp,
+								 cancellable,
+								 error)) {
+				gs_plugin_flatpak_error_convert (error);
+				return FALSE;
+			}
+		} else {
+			g_debug ("no previous %s remote to remove", remote_name_tmp);
+		}
+
+		/* remove from the store */
+		remote_id_tmp = g_strdup_printf ("%s.desktop", ref_name);
+		as_store_remove_app_by_id (self->store, remote_id_tmp);
+	}
+
 	/* install the remote, but not the app */
 	ref_file_data = g_bytes_new (contents, len);
 	xref = flatpak_installation_install_ref_file (self->installation,
@@ -2760,6 +3034,9 @@ gs_flatpak_file_to_app_ref (GsFlatpak *self,
 		as_icon_set_url (ic, ref_icon);
 		gs_app_add_icon (app, ic);
 	}
+	ref_runtime_repo = g_key_file_get_string (kf, "Flatpak Ref", "RuntimeRepo", NULL);
+	if (ref_runtime_repo != NULL)
+		gs_app_set_metadata (app, "flatpak::runtime-repo", ref_runtime_repo);
 
 	/* set the origin data */
 	remote_name = flatpak_remote_ref_get_remote_name (xref);
@@ -2792,6 +3069,10 @@ gs_flatpak_file_to_app_ref (GsFlatpak *self,
 		return FALSE;
 	}
 
+	/* get this now, as it's not going to be available at install time */
+	if (!gs_plugin_refine_item_metadata (self, app, cancellable, error))
+		return FALSE;
+
 	/* parse it */
 	if (!gs_flatpak_add_apps_from_xremote (self, xremote, cancellable, error))
 		return FALSE;
@@ -2837,7 +3118,8 @@ gs_flatpak_file_to_app (GsFlatpak *self,
 						    cancellable,
 						    error);
 	}
-	if (g_strv_contains (mimetypes_ref, content_type)) {
+	if (self->flags & GS_FLATPAK_FLAG_IS_TEMPORARY &&
+	    g_strv_contains (mimetypes_ref, content_type)) {
 		return gs_flatpak_file_to_app_ref (self,
 						   list,
 						   file,
@@ -2914,12 +3196,42 @@ gs_flatpak_store_app_removed_cb (AsStore *store, AsApp *app, GsFlatpak *self)
 	gs_plugin_cache_remove (self->plugin, as_app_get_unique_id (app));
 }
 
+const gchar *
+gs_flatpak_get_id (GsFlatpak *self)
+{
+	if (self->id == NULL) {
+		GString *str = g_string_new ("GsFlatpak");
+		g_string_append_printf (str, "-%s",
+					as_app_scope_to_string (self->scope));
+		if (flatpak_installation_get_id (self->installation) != NULL) {
+			g_string_append_printf (str, "-%s",
+						flatpak_installation_get_id (self->installation));
+		}
+		if (self->flags & GS_FLATPAK_FLAG_IS_TEMPORARY)
+			g_string_append (str, "-temp");
+		self->id = g_string_free (str, FALSE);
+	}
+	return self->id;
+}
+
 AsAppScope
 gs_flatpak_get_scope (GsFlatpak *self)
 {
 	return self->scope;
 }
 
+void
+gs_flatpak_set_flags (GsFlatpak *self, GsFlatpakFlags flags)
+{
+	self->flags = flags;
+}
+
+GsFlatpakFlags
+gs_flatpak_get_flags (GsFlatpak *self)
+{
+	return self->flags;
+}
+
 static void
 gs_flatpak_finalize (GObject *object)
 {
@@ -2927,6 +3239,13 @@ gs_flatpak_finalize (GObject *object)
 	g_return_if_fail (GS_IS_FLATPAK (object));
 	self = GS_FLATPAK (object);
 
+	if (self->changed_id > 0) {
+		g_signal_handler_disconnect (self->monitor, self->changed_id);
+		self->changed_id = 0;
+	}
+
+	g_free (self->id);
+	g_object_unref (self->installation);
 	g_object_unref (self->plugin);
 	g_object_unref (self->store);
 	g_hash_table_unref (self->broken_remotes);
